/*F File_name: 
/*F	multiplicative.c
/*
/*P Purpose:
/*P 	The purpose of this program is to take an input time series
/*P     at one time granularity and refine it into a finer-grained
/*P     time series with the same mean bit rate. The name implies
/*P     "multiplicating" the desired amount of traffic amongst the
/*P     subintervals, using multifractal wavelets. The user specifies
/*P     the number of subintervals to be created for each observation.
/*P     Uses a "" technique to generate subinterval values.
/*
/*U Usage:
/*U     cc -o multiplicative multiplicative.c -ldl -lm
/*U     multiplicative intervals mean std < input_file > output_file
/*
/*E Example:
/*E     multiplicative 10 0.5 0.5 < H8.dat > H8.mul.dat
/*
/*A Author:
/*A  	Ram Balakrishnan
/*A     (With much help from Dr.Carey Williamson)
/*
/*D Date:
/*D 	January 1999
/*
 ****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>    /* use man 3 log for the math functions */

#define RND_MAX 0x7fffffff
#define MAX_INTERVALS 1000

/* The following two values, mean and standard deviation */
/* for normal distribution, can be used to influence the */
/* short-range correlations between the values generated */
/* in the subintervals, if desired.                      */


double ceil(double x);
double erand48(unsigned short []); 
double Normal(float, float); 

/* array used for initializing seeds for Normal distribution */
unsigned short Nvar1_seed[3] = {4,7,2};
unsigned short Nvar2_seed[3] = {5,8,1};
unsigned short Nvar3_seed[3] = {9,6,3};

/* Optional debugging flag */
/* #define DEBUG 1 */

main(int argc, char **argv)
{
  int i, j, value;
  float weight[MAX_INTERVALS], normalizedweight[MAX_INTERVALS];
  float num, sum, sum2, blob, obs, nmean, nstd;

  if( argc < 4 )
    {
      printf(
	"Usage: multiplicative subintervals mean std < input_file > output_file\n"
	    );
      exit(-1);
    }

  value = atoi(argv[1]);
  if( value < 1 )
    {
      printf("Subinterval value must be positive integer!\n");
      exit(-1);
    }
  if( value > MAX_INTERVALS )
    {
      printf("Subinterval value is too large for me!\n");
      exit(-1);
    }

  nmean = atof(argv[2]);
  nstd  = atof(argv[3]);

  while( (i = scanf("%f\n", &num)) == 1 )
    {
      sum = 0.0;
      blob = 1.0 * num;
#ifdef DEBUG
      printf("multiplicating an observation of size %f into %d sub-intervals...\n",
	     blob, value);
#endif DEBUG

      for( j = 0; j < value; j++ )
	{
	  /* generate a random weight for each subinterval */
	  weight[j] = (float) Normal(nmean, nstd);

	  /* to avoid negative weightage*/
	  while(weight[j] < 0) 
	  {
	     weight[j] = (float) Normal(nmean, nstd);
	  }
  
	  sum += weight[j];
#ifdef DEBUG
	  printf("%f ", weight[j]);
	  if( j < value - 1 )
	      printf("+ ");
	}
      printf("= %f\n", sum);
#else
	}
#endif DEBUG

      sum2 = 0.0;
      /* compute normalized weights */
      for( j = 0; j < value; j++ )
	{
	  normalizedweight[j] = weight[j] / sum;
	  sum2 += normalizedweight[j];
#ifdef DEBUG
	  printf("%f ", normalizedweight[j]);
	  if( j < value - 1 )
	      printf("+ ");
	}
      printf("= %f\n", sum2);
#else
	}
#endif DEBUG

      sum2 = 0.0;
      for( j = 0; j < value; j++ )
	{
	  obs = blob * normalizedweight[j];
#ifdef DEBUG
	  printf("%f ", obs);
	  sum2 += obs;
	  if( j < value - 1 )
	      printf("+ ");
	}
      printf("= %f\n", sum2);
#else
	  printf("%f\n", obs);
	}
#endif DEBUG
    }
}
  
/***********************************************************************/
/*                 RANDOM NUMBER GENERATION STUFF                      */
/***********************************************************************/

/* Parameters for random number generation. */
#define MAX_INT 2147483647       /* Maximum positive integer 2^31 - 1 */

/* the rejection method is used for normal distribution 
   (a) Generate two uniform U(0,1) variates u1 and u2. 
   (b) Let x = -log(u1).
   (c) If u2 > e to the power -(x - 1)square by 2, go back to step
   [a].
   (d) Generate u3.
   (e) If u3 > 0.5, return (mean + (std * x)); otherwise return
   (mean - (std * x)). */

double Normal(float mean, float std)
{
  double var_u1, var_u2, var_u3;
  double x, tmp_val;
  
  var_u1 = erand48(Nvar1_seed);
  var_u2 = erand48(Nvar2_seed);
  x = -log(var_u1);
  tmp_val = exp(-((x - 1)*(x - 1)) / 2);
  while(var_u2 > tmp_val)
    {
      var_u1 = erand48(Nvar1_seed);
      var_u2 = erand48(Nvar2_seed);
      x = -log(var_u1);
      tmp_val = exp(-((x - 1)*(x - 1)) / 2);
    }
  var_u3 = erand48(Nvar3_seed);
  if(var_u3 > 0.5)
    return(mean + (std * x));
  else 
    return(mean - (std * x));
}




