/*F File_name: 
/*F	sprinkle.c
/*
/*P Purpose:
/*P 	The purpose of this program is to take an input time series
/*P     at one time granularity and refine it into a finer-grained
/*P     time series with the same mean bit rate. The name implies
/*P     "sprinkling" the desired amount of traffic amongst the
/*P     subintervals, somewhat randomly. The user specifies
/*P     the number of subintervals to be created for each observation.
/*P     Uses a "random cascade" technique to generate subinterval values.
/*
/*U Usage:
/*U     cc -o sprinkle sprinkle.c -ldl -lm
/*U     sprinkle intervals < input_file > output_file
/*
/*E Example:
/*E     sprinkle 10 < H8.dat > H8.detail.dat
/*
/*A Author:
/*A  	Carey Williamson
/*A     Modified by Ram Balakrishnan (March 1999)    
/*
/*D Date:
/*D 	June 1998
/*
 ****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "synTraff.h"

#define RND_MAX 0x7fffffff
#define MAX_INTERVALS 1000

/* The following two threshold values can be used to influence   */
/* the short-range correlations between the values generated in  */
/* the subintervals, if desired. For example, the high threshold */
/* can be used to put an upper limit on how far apart adjacent   */
/* observation values can be, while the low threshold threshold  */
/* can put a lower limit to say that adjacent values must be at  */
/* least this far apart, in U(0,1) random number space.          */
/* Be careful if using these, since they influence the marginal  */
/* distribution, the correlation structure, and the Norros       */
/* variance coefficient in subtle ways. Also, a poor choice for  */
/* the low threshold can cause an infinite loop in the program.  */
/* The default values (1.0 and 0.0) are perfectly safe to use.   */
#define LOW_THRESHOLD 0

int ET_COMMAND_sprinkle(ET_TCLARGS)
{
  int i, j, value;
  float weight[MAX_INTERVALS], normalizedweight[MAX_INTERVALS];
  float num, sum, sum2, blob, obs, HIGH_THRESHOLD;
  FILE *inFile, *outFile;

  if( argc < 4 )
    {
      Et_ResultF(interp,
		 "Usage: sprinkle HT subintervals input_file output_file\n");
      return TCL_ERROR;
    }

  HIGH_THRESHOLD = atof(argv[1]);

  value = atoi(argv[2]);
  if( value < 1 )
    {
      Et_ResultF(interp, "Sprinkle value must be positive integer!\n");
      return TCL_ERROR;
    }
  if( value > MAX_INTERVALS )
    {
      Et_ResultF(interp, "Sprinkle value is too large for me!\n");
      return TCL_ERROR;
    }

  if ((inFile = fopen(argv[3],"r")) == NULL)
  {
     Et_ResultF(interp, "Unable to open input file %s.\n",argv[2]);
     return TCL_ERROR;
  }
  outFile = fopen(argv[4],"w");


  srand(time(0));

  while( (i = fscanf(inFile,"%f\n", &num)) == 1 )
    {
      sum = 0.0;
      blob = 1.0 * num;
      for( j = 0; j < value; j++ )
	{
	  /* generate a random weight for each subinterval */
	  weight[j] = (float) rand()/RND_MAX;

	  /* extra fudging to influence short range correlations */
	  if( j > 0 )
	    {
	      while( (fabs(weight[j] - weight[j-1]) > HIGH_THRESHOLD ) ||
		    (fabs(weight[j] - weight[j-1]) < LOW_THRESHOLD) )
		{
		  weight[j] = (float) rand()/RND_MAX;
		}
	    }
	  sum += weight[j];
	}

      sum2 = 0.0;
      /* compute normalized weights */
      for( j = 0; j < value; j++ )
	{
	  normalizedweight[j] = weight[j] / sum;
	  sum2 += normalizedweight[j];
	}

      sum2 = 0.0;
      for( j = 0; j < value; j++ )
	{
	  obs = blob * normalizedweight[j];
	  fprintf(outFile,"%f\n", obs);
	}
    }
  fclose(inFile);
  fclose(outFile);
  return TCL_OK;
}


